@@ -1,15 +1,22 @@
 package File::MergeSort;
 
-our $VERSION = '1.09';
+our $VERSION = '1.10';
 
 use 5.006;     # 5.6.0
 use strict;
 use warnings;
 use Carp;
 use IO::File;
-use IO::Zlib;
 
-our @ISA = qw();
+my $have_io_zlib;
+
+BEGIN {
+    eval "require IO::Zlib";
+    unless ($@) {
+        require IO::Zlib;
+        $have_io_zlib++;
+    }
+}
 
 ### PRIVATE METHODS
 
@@ -19,9 +26,14 @@ sub _open_file {
     my $fh;
 
     if ( $file =~ /\.(z|gz)$/ ) {           # Files matching .z .gz or .zip
-	$fh = IO::Zlib->new("$file", "rb");
+
+        if ( $have_io_zlib ) {
+            $fh = IO::Zlib->new("$file", "rb");
+        } else {
+            croak "IO::Zlib not available, cannot handle compressed files. Stopping";
+        }
     } else {
-	$fh = new IO::File "< $file";
+        $fh = new IO::File "< $file";
     }
 
     return $fh || undef;
@@ -34,12 +46,12 @@ sub _get_line {
     my $line = <$fh>;
 
     if ($line) {
-	$line =~ s/\015?\012/\n/;           # This is necessary to fix CRLF problem
-	chomp $line;
-	return $line;
+        $line =~ s/\015?\012/\n/; # This is necessary to fix CRLF problem
+        chomp $line;
+        return $line;
     } else {
-	$fh->close;
-	return undef;
+        $fh->close;
+        return undef;
     }
 }
 
@@ -50,16 +62,14 @@ sub _get_index {
     # value from the line 'get_index' will return an index value that
     # can be used to compare the lines.
 
-    my $line           = shift;
-    my $index_code_ref = shift;
+    my ( $line, $index_code_ref ) = @_;
 
     my $index = $index_code_ref->($line);
 
     if ($index) {
-	return $index;
+        return $index;
     } else {
-	carp "Unable to return an index.  Continuing regardless";
-	return 0;
+        croak "Unable to return an index. Stopping";
     }
 }
 
@@ -67,101 +77,93 @@ sub _get_index {
 
 sub new {
 
-    my $class = shift;
-    croak "Expecting a class not a reference" if ref $class;
-
     ### ARGUMENTS
-    my $files_ref = shift;    # ref to array of files.
-    my $index_ref = shift;    # ref to sub that will extract index value from line
-    my $comp_ref  = shift;    # ref to sub used to compare index values
-                              #       currently not used.
+    my $class     = shift;
+    my $files_ref = shift;      # ref to array of files.
+    my $index_ref = shift;      # ref to sub that will extract index value from line
+#   my $comp_ref  = shift;      # ref to sub used to compare index values
+                                #       currently not used.
+
+    unless ( ref $files_ref eq "ARRAY" && @$files_ref) {
+	croak "Array reference of input files required";
+    }
+
+    unless ( ref $index_ref eq "CODE") {
+	croak "Code reference required for merge key extraction";
+    }
 
     ### CREATE SKELETON OBJECT
     my $self = { index      => $index_ref,
-		 comparison => $comp_ref,
-		 num_files  => 0,
-	       };
+                 num_files  => 0,
+#                comparison => $comp_ref,
+               };
 
     ### CREATE A RECORD FOR EACH FILE.
+    my @files;
     my $n = 0;
-    foreach my $file(@{$files_ref}) {
+    foreach my $file ( @{$files_ref} ) {
 
-	if ( my $fh = _open_file($file) ) {
+        if ( my $fh = _open_file($file) ) {
 
-	    $self->{files}->[$n]->{fh} = $fh;               # Store object.
-	    $self->{num_files} = $self->{num_files} + 1;    # Increase Count of open files
+            $self->{num_files}++;     # open files
+	    $files[$n]->{fh} = $fh; # Store object.
 
-	    # Get line and index for each file.
-	    $self->{files}->[$n]->{line}  = _get_line($fh);
-	    $self->{files}->[$n]->{index} = _get_index($self->{files}->[$n]->{line}, $self->{index});
-	    $n++;
+            # Get line and index for each file.
+	    $files[$n]->{line}  = _get_line($fh);
+	    $files[$n]->{index} = _get_index($files[$n]->{line}, $self->{index});
 
-	} else {
-	    carp "Error. Unable to open file, $file. Continuing regardless";
-	}
+            $n++;
+
+        } else {
+            croak "Unable to open file, $file: $!. Stopping";
+        }
     }
 
-    ### Now that the first records are complete for each file, SORT THEM.
-    ### Create a sorted arrays based on the index values of each file.
+    ### Now that the first records are complete for each file, SORT
+    ### THEM.  Create a sorted array of hashrefs based on the index
+    ### values of each file.
     ### INITIAL SORT $self->{sorted}->hash
 
-    my $array_ref = $self->{files};
-
-    my %shash;
     $n = 0;
-    foreach my $a_ref ( @{$self->{files}} ) {
-	$shash{$n} = $a_ref->{index};
-	$n++;
+    foreach my $href ( sort { $a->{'index'} cmp $b->{'index'} } @files ) {
+	$self->{sorted}->[$n++] = $href;
     }
 
-    $n=0;
-    foreach my $index ( sort { $shash{$a} cmp $shash{$b} } keys %shash ) {
-	$self->{sorted}->[$n] = $self->{files}->[$index];
-	$n++;
-    }
-
-    undef $self->{files};
-
     bless $self, $class;
-
-} # End of sub new()
+}                            # End of sub new()
 
 sub next_line {
 
     ### Main method.  This returns the next line from the stack.
 
     my $self = shift;
-
     my $line = $self->{sorted}->[0]->{line} || return undef;
 
-    # print STDERR "extracting ...", $line, "\n";  # Debug.
-
     # Re-populate LOW VALUE, i.e. $self->{sorted}->[0]
-    if ( my $newline = _get_line($self->{sorted}->[0]->{fh}) ) {
-	$self->{sorted}->[0]->{line}  = $newline;
-	$self->{sorted}->[0]->{index} = _get_index( $newline, $self->{index} );
+    if ( my $nextline = _get_line($self->{sorted}->[0]->{fh}) ) {
+        $self->{sorted}->[0]->{line}  = $nextline;
+        $self->{sorted}->[0]->{index} = _get_index( $nextline, $self->{index} );
     } else {
-	shift @{$self->{sorted}};
-	$self->{num_files}--;
+        shift @{$self->{sorted}};
+        $self->{num_files}--;
     }
 
     ### One Pass Bubble Sort of $self->{sorted}
     ### We only need to find the new positions in the stack for the
     ### new index of the file.
 
-    return $line if ($self->{num_files} <= 1);   # Abandon sorting when there is only one file left.
+    return $line if ($self->{num_files} <= 1); # Abandon sorting when there is only one file left.
 
     my $i = 0;
     while ( $self->{sorted}->[$i]->{index} gt $self->{sorted}->[$i+1]->{index} ) {
 
-	# Swap elements
-	my $place_holder = $self->{sorted}->[$i];
-	$self->{sorted}->[$i] = $self->{sorted}->[$i+1];
-	$self->{sorted}->[$i+1] = $place_holder;
-
-	$i++;
-	last if ($i > $self->{num_files} - 2);
+        # Swap elements
+        my $place_holder = $self->{sorted}->[$i];
+        $self->{sorted}->[$i]   = $self->{sorted}->[$i+1];
+        $self->{sorted}->[$i+1] = $place_holder;
 
+        $i++;
+        last if ($i > $self->{num_files} - 2);
     }
 
     return $line;
@@ -173,25 +175,30 @@ sub dump {
     # Dump the contents of the file to either STDOUT or FILE.
     # Default: STDOUT
 
-    my $self = shift;
-    my $file = shift;
+    my ( $self, $file ) = @_;
+
+    my $lines = 0;
 
     if ($file) {
-	# print STDERR "Printing to file: $file\n";  # Debug
-	open( FILE, "> $file" ) or croak "Unable to create output file $file: $!";
 
-	while ( my $line = $self->next_line ) {
-	    print FILE $line, "\n";
-	}
+        open( FILE, "> $file" ) or croak "Unable to create output file $file: $!";
 
-	close FILE or croak "Problems when closing output file $file: $!";
+        while ( my $line = $self->next_line ) {
+            print FILE $line, "\n";
+	    $lines++;
+        }
+
+        close FILE or croak "Problems when closing output file $file: $!";
 
     } else {
-	# print STDERR "Printing to STDOUT\n";  # Debug
-	while ( my $line = $self->next_line ) {
-	    print $line, "\n";
-	}
+
+        while ( my $line = $self->next_line ) {
+            print $line, "\n";
+	    $lines++;
+        }
     }
+
+    return $lines;
 }
 
 
@@ -207,7 +214,7 @@ File::MergeSort - Mergesort ordered files.
  use File::MergeSort;
 
  # Create the MergeSort object.
- my $sort = new File::MergeSort(
+ my $sort = File::MergeSort->new(
                 [ $file_1, ..., $file_n ],  # Anonymous array of input files
                 \&extract_function,         # Sub to extract merge key
                 );
@@ -220,7 +227,6 @@ File::MergeSort - Mergesort ordered files.
  # Dump remaining records in sorted order to a file.
  $sort->dump( $file );    # Omit $file to default to STDOUT
 
-
 =head1 DESCRIPTION
 
 File::MergeSort provides methods to merge and process a number of
@@ -229,8 +235,8 @@ B<pre-sorted> files into a single sorted output.
 Merge keys are extracted from the input lines using a user defined
 subroutine. Comparisons on the keys are done lexicographically.
 
-Plaintext and compressed (.z or .gz) files are catered for.
-C<IO::Zlib> is used to handle compressed files.
+If C<IO::Zlib> is installed, both plaintext and compressed (.z or .gz)
+files are catered for.
 
 File::MergeSort is a hopefully straightforward solution for situations
 where one wishes to merge data files with presorted records. An
@@ -239,6 +245,8 @@ chronologically from a cluster.
 
 =head2 POINTS TO NOTE
 
+=head3 ASCII order merging
+
 Comparisons on the merge keys are carried out lexicographically. The
 user should ensure that the subroutine used to extract merge keys
 formats the keys if required so that they sort correctly.
@@ -246,8 +254,19 @@ formats the keys if required so that they sort correctly.
 Note that earlier versions (< 1.06) of File::MergeSort preformed
 numeric, not lexicographical comparisons.
 
-=head2 DETAILS
+=head3 IO::Zlib is optional
 
+As of version 1.10 (this version) IO::Zlib is no longer a prerequisite.
+If IO::Zlib is installed, File::MergeSort will use it to handle
+compressed input files.
+
+If IO::Zlib is not installed and compressed files are specified as
+input files, File::MergeSort will raise an exception.
+
+If you do not need to process compressed files, there is no longer any
+need install IO::Zlib to use File::MergeSort.
+
+=head2 DETAILS
 
 The user is expected to supply a list of file pathnames and a function
 to extract an index value from each record line (the merge key).
@@ -319,6 +338,8 @@ Returns the next line from the merged input files.
 Reads and merges from the input files to FILENAME, or STDOUT if
 FILENAME is not given, until all files have been exhausted.
 
+Returns the number of lines output.
+
 =back
 
 =head1 EXAMPLES
@@ -372,11 +393,11 @@ FILENAME is not given, until all files have been exhausted.
  + Implement a generic test/comparison function to replace text/numeric comparison.
  + Implement a configurable record seperator.
  + Allow for optional deletion of duplicate entries.
- + Improve test suite.
+ + Ensure input is really in correct sort order - currently upto the user.
 
 =head1 EXPORTS
 
-Nothing. OO interface. See CONSTRUCTOR and METHODS
+Nothing. OO interface. See CONSTRUCTOR and METHODS.
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -389,17 +410,20 @@ it under the same terms as Perl itself.
 
 =head2 Original Author
 
- Christopher Brown E<lt>chris.brown@cal.berkeley.eduE<gt>
+Christopher Brown E<lt>chris.brown@cal.berkeley.eduE<gt>.
+
+=head2 Co-maintainer
+
+Barrie Bremner L<http://barriebremner.com/>.
 
 =head2  Contributors
 
- Barrie Bremner L<http://barriebremner.com/>
- Laura Cooney
+Laura Cooney.
 
 =head1 SEE ALSO
 
 L<perl>, L<IO::File>, L<IO::Zlib>,  L<Compress::Zlib>.
 
-L<File::Sort> as an alternative.
+L<File::Sort> or L<Sort::Merge> as possible alternatives.
 
 =cut
